@@ -308,7 +308,11 @@
 #define HAVE_STRUCT_TM_TM_ZONE 1
 #define HAVE_TM_ZONE 1
 #define HAVE_STRUCT_TM_TM_GMTOFF 1
+#define HAVE_DAYLIGHT 1
+#define HAVE_VAR_TIMEZONE 1
+#define TYPEOF_VAR_TIMEZONE long
 #define HAVE_TIMEZONE 1
+#define TIMEZONE_VOID 1
 #define NEGATIVE_TIME_T 1
 #define POSIX_SIGNAL 1
 #define RSHIFT(x,y) ((x)>>(int)(y))
