@@ -3,8 +3,8 @@
 # changes made to this file will be lost the next time ruby is built.
 
 module RbConfig
-  RUBY_VERSION.start_with?("2.2.4"[/^[0-9]+[.][0-9]+[.]/]) or
-    raise "ruby lib version (2.2.4) doesn't match executable version (#{RUBY_VERSION})"
+  RUBY_VERSION.start_with?("2.2.5"[/^[0-9]+[.][0-9]+[.]/]) or
+    raise "ruby lib version (2.2.5) doesn't match executable version (#{RUBY_VERSION})"
 
   TOPDIR = File.dirname(__FILE__).chomp!("/lib/ruby/2.2/alpha-openbsd")
   DESTDIR = '' unless defined? DESTDIR
@@ -13,8 +13,8 @@ module RbConfig
   CONFIG["MAJOR"] = "2"
   CONFIG["MINOR"] = "2"
   CONFIG["TEENY"] = "0"
-  CONFIG["PATCHLEVEL"] = "230"
-  CONFIG["INSTALL"] = '/usr/obj/ports/ruby-2.2.4/bin/install -c '
+  CONFIG["PATCHLEVEL"] = "319"
+  CONFIG["INSTALL"] = '/usr/obj/ports/ruby-2.2.5-no_ri_docs-bootstrap/bin/install -c '
   CONFIG["EXEEXT"] = ""
   CONFIG["prefix"] = (TOPDIR || DESTDIR + "/usr/local")
   CONFIG["ruby_install_name"] = "$(RUBY_BASE_NAME)22"
@@ -37,7 +37,7 @@ module RbConfig
   CONFIG["RUBY_SEARCH_PATH"] = ""
   CONFIG["UNIVERSAL_INTS"] = ""
   CONFIG["UNIVERSAL_ARCHNAMES"] = ""
-  CONFIG["configure_args"] = " '--program-suffix=22' '--with-soname=ruby22' '--with-ruby-version=minor' '--with-mantype=doc' '--enable-pthread' '--enable-ipv6' '--without-bundled-libffi' '--disable-option-checking' '--with-tcl-include=/usr/local/include/tcl8.5' '--with-tk-include=/usr/local/include/tk8.5' '--with-tcllib=tcl85' '--with-tklib=tk85' '--with-X11-dir=/usr/X11R6' '--enable-shared' '--prefix=/usr/local' '--sysconfdir=/etc' '--mandir=/usr/local/man' '--infodir=/usr/local/info' '--localstatedir=/var' '--disable-silent-rules' '--disable-gtk-doc' 'CC=cc' 'CFLAGS=-O2 -pipe' 'LDFLAGS=-L/usr/local/lib' 'CPPFLAGS=-DOPENSSL_NO_STATIC_ENGINE -I/usr/local/include' 'CXX=c++' 'CXXFLAGS=-O2 -pipe'"
+  CONFIG["configure_args"] = " '--program-suffix=22' '--with-soname=ruby22' '--with-ruby-version=minor' '--with-mantype=doc' '--enable-pthread' '--enable-ipv6' '--without-bundled-libffi' '--disable-option-checking' '--enable-shared' '--with-tcl-include=/usr/local/include/tcl8.5' '--with-tk-include=/usr/local/include/tk8.5' '--with-tcllib=tcl85' '--with-tklib=tk85' '--with-X11-dir=/usr/X11R6' '--prefix=/usr/local' '--sysconfdir=/etc' '--mandir=/usr/local/man' '--infodir=/usr/local/info' '--localstatedir=/var' '--disable-silent-rules' '--disable-gtk-doc' 'CC=cc' 'CFLAGS=-O2 -pipe' 'LDFLAGS=-L/usr/local/lib' 'CPPFLAGS=-DOPENSSL_NO_STATIC_ENGINE -I/usr/local/include' 'CXX=c++' 'CXXFLAGS=-O2 -pipe'"
   CONFIG["vendorarchdir"] = "$(vendorlibdir)/$(sitearch)"
   CONFIG["vendorlibdir"] = "$(vendordir)/$(ruby_version)"
   CONFIG["vendordir"] = "$(rubylibprefix)/vendor_ruby"
@@ -145,9 +145,9 @@ module RbConfig
   CONFIG["DTRACE"] = ""
   CONFIG["MAKEDIRS"] = "mkdir -p"
   CONFIG["MKDIR_P"] = "mkdir -p"
-  CONFIG["INSTALL_DATA"] = "/usr/obj/ports/ruby-2.2.4/bin/install -c  -m 644"
-  CONFIG["INSTALL_SCRIPT"] = "/usr/obj/ports/ruby-2.2.4/bin/install -c  -m 755"
-  CONFIG["INSTALL_PROGRAM"] = "/usr/obj/ports/ruby-2.2.4/bin/install -c -s  -m 755"
+  CONFIG["INSTALL_DATA"] = "/usr/obj/ports/ruby-2.2.5-no_ri_docs-bootstrap/bin/install -c  -m 644"
+  CONFIG["INSTALL_SCRIPT"] = "/usr/obj/ports/ruby-2.2.5-no_ri_docs-bootstrap/bin/install -c  -m 755"
+  CONFIG["INSTALL_PROGRAM"] = "/usr/obj/ports/ruby-2.2.5-no_ri_docs-bootstrap/bin/install -c -s  -m 755"
   CONFIG["SET_MAKE"] = ""
   CONFIG["LN_S"] = "ln -s"
   CONFIG["NM"] = "nm"
@@ -181,22 +181,22 @@ module RbConfig
   CONFIG["NACL_SDK_VARIANT"] = ""
   CONFIG["NACL_SDK_ROOT"] = ""
   CONFIG["NACL_TOOLCHAIN"] = ""
-  CONFIG["target_os"] = "openbsd5.9"
+  CONFIG["target_os"] = "openbsd6.0"
   CONFIG["target_vendor"] = "unknown"
   CONFIG["target_cpu"] = "alpha"
-  CONFIG["target"] = "alpha-unknown-openbsd5.9"
-  CONFIG["host_os"] = "openbsd5.9"
+  CONFIG["target"] = "alpha-unknown-openbsd6.0"
+  CONFIG["host_os"] = "openbsd6.0"
   CONFIG["host_vendor"] = "unknown"
   CONFIG["host_cpu"] = "alpha"
-  CONFIG["host"] = "alpha-unknown-openbsd5.9"
+  CONFIG["host"] = "alpha-unknown-openbsd6.0"
   CONFIG["RUBY_VERSION_NAME"] = "$(RUBY_BASE_NAME)-$(ruby_version)"
   CONFIG["RUBYW_BASE_NAME"] = "rubyw"
   CONFIG["RUBY_BASE_NAME"] = "ruby"
-  CONFIG["build_os"] = "openbsd5.9"
+  CONFIG["build_os"] = "openbsd6.0"
   CONFIG["build_vendor"] = "unknown"
   CONFIG["build_cpu"] = "alpha"
-  CONFIG["build"] = "alpha-unknown-openbsd5.9"
-  CONFIG["RUBY_PROGRAM_VERSION"] = "2.2.4"
+  CONFIG["build"] = "alpha-unknown-openbsd6.0"
+  CONFIG["RUBY_PROGRAM_VERSION"] = "2.2.5"
   CONFIG["HAVE_BASERUBY"] = "no"
   CONFIG["target_alias"] = ""
   CONFIG["host_alias"] = ""
